/*---------------------------------------------------------------------------\
| PIC18 CAN Peripheral Software for the PIC18F458
|
| File:					18F_CAN.C			Mainline code
|
| Global Routines:		CanInit()
|						CanTxIsr()
|						CanRxIsr()
|						CanTxMsg()
|						CanRxMsg()
|
| Local Routines:		None
|
| Discussion:        	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/28/2002 | Created.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes and local #defines
//------------------------------------------------------------------------

#include <p18f458.h>        // Register definitions for the PIC18F458
#include "18f_incl.h"		// Misc. definitions for this program
#include "CanModule.h"		// Includes for the CAN routines


/*---------------------------------------------------------------------------\
| Function:		CanInit
|
| Purpose:		Initialize the CAN peripheral
|
| Discussion:	Huh?
|
| Expects:		All interupts to be off and we can take as long as we want.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeCan (void)
{
	// --- Initialize the CAN peripheral

	// Reqest configuration mode
	CANCON = 0b10000000;
		  //   ||||||||
		  //   |||||||+----------- <Unimplemented on PIC>
		  //   ||||||+------------ WIN0 = 000 (RXBUF0 in SFR page)
		  //   |||||+------------- WIN1
		  //   ||||+-------------- WIN2
		  //   |||+--------------- ABAT = 0
		  //   ||+---------------- REQOP0 = 100 = Config mode
		  //   |+----------------- REQOP1
		  //   +------------------ REQOP2

	// Wait for configuration mode

	while ((CANSTATRO1 & 0x11100000) != 0x10000000)
		;

	// We want 1mbps and we have a 20MHz clock. So, we divide by 2 to
	// give us 10 CAN clock cycles (Tq's) in 1 bit. Meaning 1Tq = 100ns.
	//
	//	|------|------|------|------|------|------|------|------|------|------|
	//	|Tsync |Prop  |Seg1                       |Seg2                       |
	//                                            ^Sample = 60.0%
	//
	// The flight computer does:
	//
	//  |--------|--------|--------|--------|--------|--------|--------|--------|
	//	|Tsync   |Prop + Seg1                        |Seg2                      |
	//                                               ^Sample = 62.5%

	BRGCON1 = 0b00000000;
		  //    ||||||||
		  //    |||||||+----------- BRP0 = 000000 = 2/FOSC = 10tq/bit
		  //    ||||||+------------ BRP1
		  //    |||||+------------- BRP2
		  //    ||||+-------------- BRP3
		  //    |||+--------------- BRP4
		  //    ||+---------------- BRP5
		  //    |+----------------- SJW = 00 = 1Tq
		  //    +------------------ SJW1

	BRGCON2 = 0b10011000;
		  //    ||||||||
		  //    |||||||+----------- PRSEG0 = 000 = 1Tq
		  //    ||||||+------------ PRSEG1
		  //    |||||+------------- PRSEG2
		  //    ||||+-------------- SEG1PH0 = 011 = 4Tq
		  //    |||+--------------- SEG1PH1
		  //    ||+---------------- SEG1PH2
		  //    |+----------------- SAM = 0 = one sample
		  //    +------------------ SEG2PHTS = 1 = Use SEG2PH bits

	BRGCON3 = 0b01100010;
		  //    ||||||||
		  //    |||||||+----------- <Unimplemented on PIC>
		  //    ||||||+------------ WAKFIL = 1 = Use CAN filter on wakeup
		  //    |||||+------------- <Unimplemented on PIC>
		  //    ||||+-------------- <Unimplemented on PIC>
		  //    |||+--------------- <Unimplemented on PIC>
		  //    ||+---------------- SEG2PH0 = 011 = 4Tq
		  //    |+----------------- SEG2PH1
		  //    +------------------ SEG2PH2

	// Request operational (normal) mode

	CANCON = 0b00000000;
		  //   ||||||||
		  //   |||||||+----------- <Unimplemented on PIC>
		  //   ||||||+------------ WIN0 = 000 (RXBUF0 in SFR page)
		  //   |||||+------------- WIN1
		  //   ||||+-------------- WIN2
		  //   |||+--------------- ABAT = 0
		  //   ||+---------------- REQOP0 = 000 = Normal mode
		  //   |+----------------- REQOP1
		  //   +------------------ REQOP2

	// Wait for normal mode

	while ((CANSTATRO1 & 0x11100000) != 0x00000000)
		;
}


FROM OLD TIMER2 ROUTINE:


	if (cCount >= 250)
	{
		cCount = 0;

		// Toggle the RD4 LED on and off

		if (PORTDbits.RD4 == 1)
			PORTDbits.RD4 = 0;
		else
		{
			PORTDbits.RD4 = 1;

			// Send a CAN message if we just turned the LED on

			if (TXB0CONbits.TXREQ == 0)
			{
				TXB0SIDH = 0x80;		// CANID = 0x10000000000
				TXB0SIDL = 0x00;
				TXB0D0 = 0x00;			// Set up the data bytes
				TXB0D1 = 0x11;
				TXB0D2 = 0x22;
				TXB0D3 = 0x33;
				TXB0D4 = 0x44;
				TXB0D5 = 0x55;
				TXB0D6 = 0x66;
				TXB0D7 = 0x77;
				TXB0DLC = 0x00001000;	// Send 8 data bytes

				TXB0CONbits.TXREQ = 1;	// Request to send
			}
		}
	}
	else
		++cCount;

