/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				main.c
|
| Description:		Mainline routines.
|
| Global Routines:	Main()
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <p18f458.h>    // Register definitions for the PIC18F458

#include "project.h"	// Includes for the project (Globals + misc)
#include "main.h"		// Includes for the mainline
#include "interrupts.h"	// Includes for interrupt routines and variables
#include "exceptions.h"	// Includes for exception handling
#include "ports.h"		// Includes for the port peripherals
#include "timer2.h"		// Includes for the Timer2 peripheral
#include "serial.h"		// Includes for the serial peripheral (UART )
//#include "can.h"		// Includes for the CAN peripheral


// Note that we'll also need the linker file "18F458.LKR"
// which should be added to the Project: Edit Project dialog via
// "Add Node". Be SURE to add files in the MCC18 directory (NOT
// from the MPLAB directory).




/*---------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	Doesn't do much, huh?
|
| Expects:		Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{

	// Initialize everything and its brother

	Initialize();


	// Loop for ever...and......ev....er....and.......ev............er.

	while (TRUE)
		/* Do absolutely positively nothing */ ;
}




/*---------------------------------------------------------------------------\
| Function:		Initialize
|
| Purpose:		Initialize everything. Really.
|
| Discussion:	Did I mention initialization?
|
| Expects:		to be called after a RESET - eg, registers reset to known
|				values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

	/*
		EVENTUALLY: Check for a reset: If there hasn't been a reset,
		then something has gone horribly wrong. Do a SOFTWARE RESET
		here to put everything in a known state.
	*/

	// Get the outputs stabilized FIRST.

	InitializePorts();

	// Turn on serial transmitter and receiver. Turn on receiver interrupts
	// but leave the transmitter interrupts off (we'll turn them on when
	// there's actual data to send over the port). Make both interrupts
	// low priority, and se the bit rate to 9600bps.

	InitializeSerial (	SERIAL_TRANSMIT_ENABLE
						| SERIAL_TRANSMIT_DISABLE_INTS
						| SERIAL_TRANSMIT_INTS_LOWPRIORITY
						| SERIAL_RECEIVE_ENABLE
						| SERIAL_RECEIVE_ENABLE_INTS
						| SERIAL_RECEIVE_INTS_LOWPRIORITY
						| SERIAL_BITRATE_9600 );

	// Turn on timer2, HP interrupts @ 1KHz

	InitializeTimer2 (	TIMER2_ENABLE
						| TIMER2_ENABLE_INTS
						| TIMER2_INTS_HIGHPRIORITY
						| TIMER2_1KHZ );

	// Turn on CAN

//	InitializeCan();

	// Initialize variables, if necessary, before start up

	InitializeVariables();

	//
	// Initialize interrupts **AFTER** everything has been initialized.
	//

//	InitializeInterrupts();
}





/*---------------------------------------------------------------------------\
| Function:		InitializeVariables
|
| Purpose:		Initialize all mainline variables
|
| Discussion:	Sure.
|
| Expects:		To be called before interrupts enabled, although it's not
|				a requirement.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/


void	InitializeVariables (void)
{
	/* Do nothing */;
}



